/* Header file for UHFA.C, ver 1.0 26 Feb 97   RSL */
/* Revised modes w/LHL7, PUA43   23 Jan 00     RSL */

#define TRUE  1
#define FALSE 0
#define COM_PORT	COM2
#define BAUD_RATE	9600
#define COM_SETTINGS	(BITS_8 | STOP_1 | NO_PARITY)
#define ESC_CHAR	'$'	/* Was '1' */
#define SPACE		32
#define HYPHEN		45

/* Some numbers within '120 dB' of bottom */ 
#define D_TINY		1.0E12*DBL_MIN
/*   and of the top */
#define D_VBIG		1.0E-12*DBL_MAX

/* The following are the outlines of the boxes in screen coordinates */
#define X1L		128
#define X1R		639
#define Y1T		1
#define Y1B		129
#define X2L		128
#define X2R		639
#define Y2T		147
#define Y2B		479
#define S_METER_POS	197

#define BOTTOM_LINE	33
#define NO_CLIP		0
#define DO_CLIP		1

#define	CW		0
#define USB		1
#define LSB		2
#define FM		3
#define LHL7            4
#define PUA43           5
#define EME2		6
#define LTI		7
#define AM		8

/* For display_width:   */
#define FULL_FREQ	0
#define HALF_FREQ	1
#define LST_GMT		8

#define IO_LPT1		0X378
#define IO_LPT2		0X3BC

#define R1MHZ		0X08191UL	/*  19 bit R counter, P & C=1, 1MHz */
#define R5MHZ		0X087D1UL	/*   5 MHz ref freq, 1st LO synth */
#define R10MHZ		0X08FA1UL	/*   and 10 MHz */

#define	INIT_CW		0		/* Initialize Morse code routine */
#define	ADD_CH		1		/* Put char into cw buffer */
#define SEND_CW		2		/* Send a character if available */

#define UBLACK		0
#define UGREENBLACK	1
#define	UOLIVE		2
#define UGREENBLUE	3
#define UBLUE		4
#define ULIGHTPURPLE	5
#define UPURPLE		6
#define UPURPLERED	7
#define URED		8
#define ULIGHTRED	9
#define UORANGE		10
#define UBEIGE		11
#define UHEAVYYELLOW	12
#define UYELLOW		13
#define UPALEYELLOW	14
#define UWHITE		15

/* FLOW and FHIGH are spectral bin limits used to measure average spectrum
 * values for normalization and for determining analog gain changes.  Avoid too
 * low a FLOW due to 60 Hz interference, etc.
 */
#define	FLOW		50
#define	FHIGH		250

extern void update_data(void);
extern void prep_display(void);
extern void print_stars(void);
extern void draw_grid(void);
extern void sa_graph(void);
extern void mark_frequency(int);
extern void t_r_toggle(void);
extern void new_freq(void);
extern void set_afgain(int);
extern void set_rfgain(void);
extern void set_mike_gain(void);
extern void set_fm_squelch(void);
extern void set_cwoffset(void);
extern void set_eme_doppler(void);
extern void show_er(void);
extern void set_fsk(int);
extern void set_xmit_power(void);
extern void set_contrast(void);
extern void set_brightness(void);
extern void set_window(void);
extern void set_affilt_sw(void);
extern void set_affilt_select(void);
extern void set_delay_right(int);
extern void filter_design(void);
extern void select_denoise(void);
extern void set_denoise(void);
extern void annotate_af_filt(void);
extern void set_dot(void);
extern void set_display_width(void);
extern void set_disp_offset(int);
extern void set_db_div(int);
extern void set_noncohave(int);
extern void set_mode(int);
extern void set_code_nx2(void);
extern void set_input(void);
extern void set_8_16(int);		/* 0=set to 8 bit, 1=toggle 8/16 */
extern void set_data_save(void);
extern void disp_toggle(void);
extern void normalize_plot(int);	/* 0=clear, 1=15 second, 2=10 min */
extern void normalize(int);		/* Do the normalization calcs, 256 or 512 */
extern void lt_normalize(void);
extern void dofreqplot(int);
extern void fsk2sec(void);		/* Sends 2 sec fsk tone */
extern void set_clock(int);		/* Manual set of DSP clock (NOT PC clock) */
extern long int  update_clock(int);	/* Set/get start of next second */
extern void diagnostics(int);
extern void freq_data(int, long int, long int, long int, long int, long int,
               unsigned int);
extern void initialize(void);
extern void presets(void);
extern void do_transmit(void);
extern void eme_timing(void);		/* Poles eme_state after times all */
extern void lhl7_timing(void);
extern void do_auto_disp(int k);
extern int  read_config_file(void);
extern void save_state(void);
extern void save_lt_pwr(void);
extern void new_band(int);
extern void rit_toggle(void);
extern void display_summary(char*, char*);
extern void print_time(void);
extern void send_shift_data(unsigned long);  /* Load shift regs and synth from dsp */
extern void tiny_time(int, int, int);
extern void tiny_char(int, int, int);
extern void micro_char(int, int, int);
extern void point1(int, int);
extern void point5(int, int);
extern void beep(int);
extern int  near_int(double);
extern void bar_graph(int, int, unsigned int);
extern int  get_kbd_int(void);
extern int  keyident(void);
extern void get_mem(void);
extern void set_agc(int);
extern void set_cw_speed(void);
extern void draw_panel(void);
extern double p2db(double);
extern double upow10(double);
extern void get_dsp_ptrs(void);
extern void get_dsp_mem(int what, int* tbl, int item, int offset);
extern void set_dsp_mem(int what, int adr, long int data);
extern int  get_status_bytes(int type, double timeout);
extern double update_moon(struct tm *ptm, int do_now);
extern void update_sun(struct tm *ptm, int do_now);
extern void set_cur_loc(int);
extern void set_bins(void);
extern void do_t_error(int, char*, int);

/*************************************************************************/

struct transverter_info
	{
	char	name[12];		/* Identifier, printed in upper-left */
	int	lo_type;		/* 0=open loop, 1=ref locked, 2=counter
					   Only 0 presently implemented */
	int	sb;			/* -1 if i-f=rf-lo, else 1 */
	double	lo_freq;		/* lo freq in Hz */
	int	chain_mult;		/* For counter, if sub-multiple of lo*/
	char	init_str[20];		/* Serial string for transverter init
					   or the string 'NULL' if not used.
					   Only NULL presently implemented */
	double	gain_db;		/* Used to make dBm, like 23.7 */
	int	control;		/* Control options, not used */
	unsigned bus_adr;		/* For controlling tvtr's */
	double	last_tfreq;		/* Where last tuned */
	int	last_mode;		/* Last mode used */
	};

struct rf_gn
	{
	double	dbgain;			/* 0.0 or less, 0.0 for [0] entry */
	int	gainbyte;		/* Gets ORed with shift_reg */
	int	dbdisplay;		/* 0 to 100, 100 for [0] entry (dB) */
	};

struct filter_info
	{
	char	name[7];		/* Identifier, printed on screen */
	char	file_name[31];		/* Extensions .fil and .dat */
	int	fil_type;		/* 0=Misc 1=Kaiser, 2=Parks-Mc,
					 * 3=McClure match, 4=Equalizer */
	double	param1;			/* Side lobe dB for Kaiser */
	double	param2;			/* NU */
	int	param3;			/* NU */
	int	fclow;			/* (0, 4800) */
	int	fchigh;			/* (0, 4800) */
	int	fil_gain;		/* Shifts inherent in coefs (0,15) */
	};

extern char		*big_area_ptr1;
extern char		*big_area_ptr2;
extern char		cc[];		/* General purpose string area */
extern int		contrast;
extern double		dis_gain;
extern int		bright;
extern double		dis_offset;
extern int		power_exp;  /*Power of 2 that dsp data is divided by
				      before integration; must be (0, 255).  */
extern int		scroll_line;
/* T/R is controlled either by PTT on rf board or by 'Home'. The following
   three variables allow either but not both at any time.   */
extern unsigned int	transmit;	/* 1=transmit, 0=receive */
extern unsigned int	ptt;		/* PTT: 1=transmit, 0=receive */
extern unsigned int	home_tr;	/* 'Home' key: 1=transmit, 0=receive */
extern unsigned int	cw_on;		/* 0=cw off, 1=cw on */
extern int	rfgain;			/* (0, 6) for 0, -6, ... -36 dB */
extern int	xmit_power;
extern int	mike_gain;
extern int	afgain;
extern int	fm_afgain;
extern int	cwoffset;
extern int	fsk;			/* Modulation tone for LHL7, PUA43 */
extern int	sidetone_lev;
extern int	fm_squelch;
extern int	noncohave;
extern int      d_noncoh;
extern unsigned int fft_fcn;
extern int	fft_bw;
extern int	do_window;
extern double win_adj;			/* Correctn for Hamming/BH-92 window */
extern int    do_af_filt;		/* 0=no filt, 1=af filt on */
extern int	af_coef;		/* 0=dsp coefs0, 1=coefs1 */
extern int    do_denoise;		/* 0=no denoise, 1=denoise on */
extern int	denoise_level;
extern int	denoise_decay;
extern int      do_LMS;
extern int      autonotch_level;
extern int	delay_state;		/* 0=No binaural delay */
extern int	delay_right;		/* Power of 2 x 20.8usec on right ch */
extern int	agc_decay;
extern int	dot_type;
extern int	display_width;
extern int	bin_low;		/* Lowest bin for marker */
extern int	db_div_set;		/* 0=10db/div, 1=5db, 2=2db, 3=1db */
extern double	kdb_div;		/* 80 div/256 values */
extern double	koffset;		/* Display offset in dB */
extern int	mode;
extern int	narrow;
extern int	binaural;
extern int	iq;
extern int	bits_8_16;		/* 0=8 bit log data, 1=16 bit */
extern int	save_8_16;
extern int	input_if_af;		/* Start as rf in */
extern int	hardware;		/* RF Hardware present */
extern int	normalize_on;		/* 0=off, 1=on, >1 is countdown */
extern int	auto_disp_on;		/* 0=off, 1=on  */
extern int	rit;
extern int	rit_save;
extern int	rit_on;
extern int	band;				/* Band 1 = 144 MHz */
extern long	tfreqL;
extern struct	 palettetype  upalette;
extern int	startup_detail;		/* 0=No detail, 1=Show diagnostics
					   Set =1 in case .cfg missing */
extern int	waterfall;		/* 0=Fancy freq dial, 1=Waterfall */
extern int	time_type;		/* 0=GMT, 1=local */
extern unsigned char code_id[];        /* ID when sending fsk code */
extern int	code_nx2;		/* Number of 2 sec per tone */
extern double  millisec;		/* Corrected count of millisec */
extern double  millisec_offset;	/* Used in clock setting to line up
					   ticks */
extern double  clock_cor_factor;	/* Relative speed of PC clock */
extern long int  dsp_clock_base;	/* Offset to make millisec valid */
extern long int  cl_ms;		/* Results from last clock() */
extern int	clock_been_set;		/* Error for codes, if not set */
extern int	quit_save_state;	/* TRUE=save state on quit() */
extern int	beep_on_error;		/* When error codes are displayed */
extern int	error_on_tr;			/* Prevent error after T/R change */
extern struct	transverter_info tvtr[];
extern unsigned int	dsp_deltap_lo;	/* Phase incr dsp, low 8 bits */
extern unsigned int	dsp_deltap_hi;	/* Phase incr dsp, high 8 bits */
extern int	dsp_status[];
extern int	dsp_xmit_status[];
extern double	ave_spectral_pwr;	/* Ave if color agc on */
extern double	ref_offset;		/* Correction factor for freq ref */
extern int	integrate_count;	/* Varies from 0 to noncohave/6  */
extern int	contrast_line_count;	/* Display contrast line if >0 */
extern int	cw_speed;		/* Index of speed (0, 25) */
extern int	display_error_count;
extern int	diag_mode;		/* 0=none, 1=text, 2=bargraphs
					   3=Moon/Sun data */
extern int	do_data_save;		/* 0=no save to file, 1=save */
extern unsigned int  com_port;

extern struct  palettetype  savpalette;

extern int c_background;
extern int c_outline;
extern int c_outline_bold;
extern int c_outline_bright;
extern int c_text;
extern int c_text_bold;
extern int c_text_bright;
extern int c_text_note;
extern int c_plot_bold;
extern int c_plot_bright;
extern int c_plot_marker;
extern int c_bar_dark;
extern int c_bar_low;
extern int c_bar_medium;
extern int c_bar_high;

extern double   bin_width[];
extern double   noise_bw[];

extern int	incount;
extern double	fgain[];		/* 'Exact' gain value */
extern int	fgainl[];		/* Low 8 bits of gain value */
extern int	fgainh[];		/*   and high 8 bits */
extern int	raw_data[];
extern double*	dnorm;
extern double*	lt_dnorm;
extern double	normalize_num;
extern double   eme2_te;		/* Effective Noise Temp Kelvins */
extern double*	pwrdat;			/* An array of 515 for power from
					   dsp fft.  Uses calloc() */
extern double*	pd16;			/* An array of power for 16 bit data */
extern double	pave;
extern double	paveave;		/* Rng ave of pave */
extern double*  lt_pwrdat;		/* 'Perpetual' power accumulation */
extern double	lt_max;			/* Limits of displayed integrated */
extern double	lt_min;			/*   power for scaling display */
extern int	lterm_dorandom;
extern double*	power10;		/* 256 entry table for converting log
					   to power. Uses calloc() */
extern FILE*	data_file;
extern FILE*	config_file;
extern FILE*    license_file;
extern struct   rf_gn  rfg[];
extern double	gain_adj;		/* dB factor to correct pwr disp */
extern int	screen_save;
extern struct	eenv  cur_env;		/* Cur enviornment:day, nutation, etc*/
extern int	ms_available;		/* TRUE if PC has millisec clock */
extern int	ut_available;		/* 0 if not avail, else 1 */
extern double	ut;			/* UT for moonsun calculations */
extern struct   tm    tdt;		/* TDT for ephemeris */
extern double	f_eme;			/* Dopplr Fr (audio processing only) */

/* For LHL7 mode, these variables:  */
extern struct	lhl  lc0;		/* Info on LHL codes from u_code */
extern int	lhl_bins;		/* 1, 3, 5, 7, or 9 to look over */
extern int	lhl_peak_ave;		/* Look for peak (=0) or ave (=1) */
extern int	lhl7_2line;		/* 0-->1 big line, 1-->2 small lines */

/* For PUA43, we need: */
extern int	pua14_28;
extern int	pua43_dorandom;
extern double	pua43_noisedecay;
extern double	pua43_colorscale;
extern int	code_min;		/* Minute of day (0, 1439) */
extern int*	pua43_rnd;		/* Array of 1440 random num */
extern char	pua43_xstring[];	/* 14 or 28 characters to be sent */
/* The following two parameters specify the PUA43 frequency arrangement
 * as a function of the fft_bw parameter.     */
extern double	df[];			/* Bin size, Hz, [fft_bw] */
extern int	b0[];			/* Base bin for PUA43, [fft_bw] */

/* Moon-Sun: */
extern struct	location loc[];
extern int	cur_loc;
extern struct	eenv cur_env;		/* Cur enviornment:day, nutation, etc*/
extern int	ut_available;		/* 0 if not avail, else 1 */
extern double	ut;			/* UT for moonsun calculations */
extern double   dut1;			/* UT1-UTC if known, else 0.0 */
extern struct  tm   tdt;		/* TDT for ephereris calculations */
extern double	f_eme;			/* Dopplr Fr (audio processing only) */

/* For EME2 mode we need:    */
extern long int    eme_pulse_count;	/* The number of 5 second periods */
extern double      eme_pwr_sum[];	/* Long term pwr +/- 10 bins */
extern int         eme_freq_ctrl;	/* 0=fixed, 1=low noise, 2=random */
extern int         eme_doppler;		/* 0=no correct, 1=track doppler */
extern int         eme_state;		/* 0=clear, 1=standby, 2=active */
extern int         eme_record;		/* 0=off, 1=21 freq every 5 sec */
extern int         eme_delay;
extern int         eme2_format;		/* Bit  0: 1=doppler correct
					 *      1: 1=use random freq
					 *      2: 1=use auto noise blanker */
extern double	   eme_dop1, eme_dop2;	/* Due to E-M move, rotation */
extern double	   eme_dop3, eme_dop4;	/* Due to RA move, Dec move */
extern double      eme_freq_range;	/* Spread of EME2 frequencies */
extern double      code_nb_ratio;	/* Max noise/ave noise (NBlank) */
extern double      min_lt_dd;		/* Lowest div/db for "yelow" displays*/
/* Parameters for USB long term integration */
extern int         lterm_int;		/* TRUE or FALSE */
extern double      lterm_int_flow;	/* in Hz */
extern double      lterm_int_fhigh;
extern double      lterm_int_center;
extern double      lti_freq_range;	/* Random spread LTI */
extern double      lti_frandom;          /* Actual current random delta f */

/* Bin numbers for lomg term integration in USB and EME2: */
extern int         lt_i1, lt_i2, lt_i3, lt_i4, lt_i5;

extern unsigned int mat_err;		/* Matrix calculation errors */

extern long int    random_freqL;

extern struct filter_info filt[];	/* Audio filters 0 to 7 */
extern double  flow;			/* Parameters for filter design */
extern double  fhigh;
extern double  kaiser_db;
extern int     fil8_use;
extern char    fil8_name[];
/* Parameters for receive equalizer that is part of filter design: */
#define	NFRB	15			/* Break frequencies (not progrmabl) */
extern double feqr[];			/* Equalizer break frequencies */
/* Relative equalizer gains in dB (program from .cfg file): */
extern double aeqrdb[];

extern void*       current_info_box;    /* 0 if no box, otherwise ptr */

extern int dsp_ptr[];			/* Table of pointers in DSP pm() */
extern int dsp_data_index;		/* Keeps track of data from DSP */
extern int table_adr;			/* DSP adr of ptr table, =0 if error */

extern unsigned int errcode;
extern int   kbd_alt1;
extern int   kbd_alt2;
extern int   kbd_alt3;

extern struct lbox_info lb_freq; 		/* Freq info box */
extern struct lbox_info lb_eme2;		/* EME2 data summary box */
extern struct lbox_info lb_lhl7;		/* and for LHL7 */
extern struct lbox_info lb_pua43;		/* and for PUA43 */
extern struct lbox_info lb_lterm_int;		/* and for LTI */
extern void* current_info_box;			/* 0 if no box, else ptr */
